//
//  Copyright (C) 2015 Artem Anufrij <artem.anufrij@live.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Authored by:    Artem Anufrij <artem.anufrij@live.de>

namespace Footnote.Dialogs {

    public class SettingsDialog : Gtk.Dialog {

        public Gtk.FileChooserButton    db_filechooser;
        private Gtk.Stack               main_stack;
        private Gtk.StackSwitcher       main_stackswitcher;

        private string                  db_location;

        public SettingsDialog (Gtk.Window? parent) {

            if (parent != null)
                set_transient_for (parent);

            this.destroy.connect (() => {
                if (db_filechooser.get_filename () != db_location)
                    Footnote.DataBase.DatabaseManager.get_default ().move_database (db_filechooser.get_filename ());
            });

            this.db_location = Footnote.Settings.get_default ().database_location;

            this.main_stack = new Gtk.Stack ();
            this.main_stackswitcher = new Gtk.StackSwitcher ();
            this.main_stackswitcher.set_stack (main_stack);
            this.main_stackswitcher.halign = Gtk.Align.CENTER;

            this.build_ui ();
        }

        private void build_ui () {
            this.title = _("Settings");
            this.set_default_size (630, 330);
            this.window_position = Gtk.WindowPosition.CENTER;
            this.resizable = false;
            this.deletable = false;

            // General Tab
            this.main_stack.add_titled (get_general_box (), "general", _("General"));

            // Close Button
            var close_button = new Gtk.Button.with_label (_("Close"));
            close_button.clicked.connect (() => {
                this.destroy ();
            });

            // Button Box
            var button_box = new Gtk.ButtonBox (Gtk.Orientation.HORIZONTAL);
            button_box.set_layout (Gtk.ButtonBoxStyle.END);
            button_box.pack_end (close_button);
            button_box.margin_right = 12;

            // Main Grid
            Gtk.Grid main_grid = new Gtk.Grid ();
            main_grid.attach (this.main_stackswitcher, 0, 0, 1, 1);
            main_grid.attach (this.main_stack, 0, 1, 1, 1);
            main_grid.attach (button_box, 0, 2, 1, 1);

            ((Gtk.Container) get_content_area ()).add (main_grid);
        }

        Gtk.Widget get_general_box () {
            int row = 0;

            var general_grid = new Gtk.Grid ();
            general_grid.margin = 12;
            general_grid.row_spacing = 5;
            general_grid.column_spacing = 5;
        
            // Label Layout
            var label = new Gtk.Label (_("Layout:"));
            add_section (general_grid, label, ref row);
            
            // Label Monthbar
            var monthbar = new Gtk.Switch ();
            Footnote.Settings.get_default ().schema.bind("show-month-bar", monthbar, "active", SettingsBindFlags.DEFAULT);
            label = new Gtk.Label (_("Show month bar:"));
            add_option (general_grid, label, monthbar, ref row);

            // Label FileChooser
            label = new Gtk.Label (_("Database location:"));
            add_section (general_grid, label, ref row);

            // FileChooser
            db_filechooser = new Gtk.FileChooserButton (_("Select Folder…"), Gtk.FileChooserAction.SELECT_FOLDER);
            db_filechooser.hexpand = true;
            db_filechooser.set_current_folder (db_location);
            db_filechooser.current_folder_changed.connect (() => {

            });
            general_grid.attach (db_filechooser, 0, ++row, 3, 1);

            return general_grid;
        }

         void add_section (Gtk.Grid grid, Gtk.Label name, ref int row) {
            name.use_markup = true;
            name.set_markup ("<b>%s</b>".printf (name.get_text ()));
            name.halign = Gtk.Align.START;
            grid.attach (name, 0, row, 2, 1);
            row ++;
        }

        void add_option (Gtk.Grid grid, Gtk.Widget label, Gtk.Widget switcher, ref int row) {
            label.hexpand = true;
            label.halign = Gtk.Align.END;
            label.margin_left = 20;
            switcher.halign = Gtk.Align.FILL;
            switcher.hexpand = true;
            
            if (switcher is Gtk.Switch || switcher is Gtk.Entry) { /* then we don't want it to be expanded */
                switcher.halign = Gtk.Align.START;
            }
            
            grid.attach (label, 0, row, 1, 1);
            grid.attach_next_to (switcher, label, Gtk.PositionType.RIGHT, 1, 1);
            row ++;
        }
    }
}
