//
//  Copyright (C) 2012 Andrea Basso
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

namespace Footnote.Widgets {
    /**
     * class representing an export dialog for both notes and notebooks
     **/
    public class CustomPrint : Gtk.PrintOperation {
        int FONT_SIZE = 12;
        int LINE_HEIGHT = 17;
        int MARGIN_LEFT = 20;
        int MARGIN_TOP = 40;

        string content = "";
        string[] pages_text = {};
        bool[] prefs;
        Cairo.Context context;
        Pango.Layout layout;
        Pango.FontDescription desc;

        public CustomPrint (Gee.ArrayList<Footnote.Objects.Note> notes, bool[] preferences) {
            prefs = preferences;
        
            begin_print.connect ((print_context) => {
                context = print_context.get_cairo_context ();
                layout = Pango.cairo_create_layout (context);
                desc = new Pango.FontDescription ();
                
                desc.set_family ("Open Sans");;
                desc.set_absolute_size (Pango.SCALE*FONT_SIZE);
                layout.set_wrap (Pango.WrapMode.WORD_CHAR);
                layout.set_width (Pango.SCALE*((int)print_context.get_width ()));
                int height = (int)print_context.get_height ();
                layout.set_height (Pango.SCALE*height);
                
                context.move_to (MARGIN_LEFT, MARGIN_TOP);
                layout.set_font_description (desc);
                
                foreach (var n in notes) {
                    if (prefs[0])
                        content += n.title.text + "\n";
                    if (prefs[2])
                        content += n.get_time_label () + "\n\n";
                    if (prefs[1])
                        content += n.content.text + "\n ~#end#~\n";
                }
                
                string written_lines = "";
                int pages = 1;
                int max_lines_per_page = height/LINE_HEIGHT;
                int line_count = layout.get_line_count ();
                int lines_to_end = 0;
                string[] lines = content.split ("\n");

                foreach (string line in lines) {
                    stdout.printf("Lines: %d %d\n", max_lines_per_page, lines.length);

                    written_lines += line + "\n";

                    // Be sure the complete note is on a new page!
                    // Else there will be all kinds of weird formatting errors.
                    if ("~#end#~" in line) {
                        int compare_count = max_lines_per_page - line_count;
                        if (lines_to_end > compare_count) {
                            for (int i = 0; i < compare_count; i++) {
                                written_lines += "\n";
                            }
                        }
                        lines_to_end = 0;
                    } else {
                        lines_to_end++;
                    }
                    
                    layout.set_text (written_lines, -1);
                    line_count = layout.get_line_count ();
                    if (line_count >= max_lines_per_page) {
                        pages_text += written_lines;
                        written_lines = "";
                        pages++;
                    }
                }

                pages_text += written_lines;
                set_n_pages (pages);
            });
            
            bool title = true;
            bool date = false;
                
            draw_page.connect ( (operation, print_context, page_num)=> {
                layout.set_wrap (Pango.WrapMode.WORD_CHAR);
                layout.set_width (Pango.SCALE*((int)print_context.get_width ()));

                string text = "";
                string body = "";
                foreach (string line in pages_text[page_num].split ("\n")) {
                    if (line != "") {
                        if (title) {
                            date = true;
                            title = false;
                            if (prefs[0])
                                text += @"<span font='Open Sans 9'><b>$line</b></span>\n";
                        } else if (date) {
                            date = false;
                            if (prefs[2])
                                text += @"<span color='#7F7F7F' font='Open Sans 7'>$line</span>\n";
                        } else {
                            if ("~#end#~" in line) {
                                text += @"<span font='Open Sans 9'>$body</span>\n\n";
                                body = "";
                                title = true;
                            } else if (prefs[1]) {
                                body += line + "\n";
                            }
                        }
                    }
                }
                
                if (!(title || date))
                    text += @"<span font='Open Sans 9'>" + body.replace ("<", "&#60;").replace (">", "&#62;") + "</span>\n\n";
                layout.set_markup (text, -1);
                Pango.cairo_show_layout(context, layout);
            });
        }
    }
}
