/*
 *  Copyright (C) 2014-2015 Elementary Developers
 *
 *  This program or library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301 USA.
 *
 *  Authored by:    Artem Anufrij <artem.anufrij@live.de>
 */

namespace Footnote.Widgets {

    public class EditView : Gtk.EventBox {

        public Gtk.TextView     title;
        public Gtk.TextView     content;
        public Gtk.Toolbar      editbar;
        private Gtk.ScrolledWindow content_scroll;
        private Gtk.ToggleToolButton bold;
        private Gtk.ToggleToolButton italic;
        private Gtk.ToggleToolButton strike;
        private Gtk.ToggleToolButton underline;
        private Gtk.ToolButton mark;
        private Granite.Widgets.ModeButton mark_color_buttons;
        private Gtk.Popover mark_popover;
        private Gtk.ToolButton revert;

        private bool applying_style = false;

        public signal void close (bool reverse=false);
        public signal void open  ();

        public Footnote.Objects.Note current_note;

        FootnoteApp app;

        public EditView (FootnoteApp app) {

            this.app = app;

            var container = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);

            this.visible_window = false;
            container.get_style_context ().add_class (Gtk.STYLE_CLASS_VIEW);

            editbar = new Gtk.Toolbar ();
            editbar.get_style_context ().add_class ("secondary-toolbar");

            title = new Gtk.TextView ();
            title.override_background_color (Gtk.StateFlags.NORMAL, {0, 0, 0, 0});

            content = new Gtk.TextView ();
            content.override_background_color (Gtk.StateFlags.NORMAL, {0, 0, 0, 0});

            content_scroll = new Gtk.ScrolledWindow (null, null);

            var undo = new Gtk.ToolButton (null, null);
            undo.tooltip_text = _("Undo");
            undo.icon_name = "edit-undo";

            var redo = new Gtk.ToolButton (null, null);
            redo.tooltip_text = _("Redo");
            redo.icon_name = "edit-redo";

            bold = new Gtk.ToggleToolButton ();
            bold.tooltip_text = _("Set the selected text to bold");
            bold.icon_name = "format-text-bold";
            bold.toggled.connect (() => {
                if (applying_style == false) {
                    current_note.tag ("bold", bold.active);
                    current_note.changed ();
                }
            });

            italic = new Gtk.ToggleToolButton ();
            italic.tooltip_text = _("Set the selected text to italic");
            italic.icon_name = "format-text-italic";
            italic.toggled.connect (() => {
                if (applying_style == false) {
                    current_note.tag ("italic", italic.active);
                    current_note.changed ();
                }
            });

            underline = new Gtk.ToggleToolButton ();
            underline.tooltip_text = _("Underline the selected text");
            underline.icon_name = "format-text-underline";
            underline.toggled.connect (() => {
                if (applying_style == false) {
                    current_note.tag ("underline", underline.active);
                    current_note.changed ();
                }
            });

            strike = new Gtk.ToggleToolButton ();
            strike.tooltip_text = _("Strike though the selected text");
            strike.icon_name = "format-text-strikethrough";
            strike.toggled.connect (() => {
                if (applying_style == false) {
                    current_note.tag ("strikethrough", strike.active);
                    current_note.changed ();
                }
            });

            // MAKR
            mark = new Gtk.ToolButton (null, null);
            mark.tooltip_text = _("Mark the selected text");
            mark.icon_widget = new Gtk.Image.from_file (Constants.PKGDATADIR + "/format_text_highlight.svg");
            mark.clicked.connect (() => {
                mark_popover.show_all ();
            });

            mark_color_buttons = new Granite.Widgets.ModeButton ();
            mark_color_buttons.append_pixbuf(get_color_dot (218, 77, 69, 1));    // RED
            mark_color_buttons.append_pixbuf(get_color_dot (243, 115, 41, 1));   // ORANGE
            mark_color_buttons.append_pixbuf(get_color_dot (251, 210, 93, 1));   // YELLOW
            mark_color_buttons.append_pixbuf(get_color_dot (179, 207, 59, 1));   // GREEN
            mark_color_buttons.append_pixbuf(get_color_dot (71, 168, 233, 1));   // BLUE
            mark_color_buttons.append_pixbuf(get_color_dot (195, 92, 194, 1));   // PURPLE
            mark_color_buttons.mode_changed.connect ((widget) => {
                if (applying_style == false) {

                    current_note.clear_marked_color ();

                    switch (mark_color_buttons.selected) {
                        case 0:
                            current_note.tag ("marked_red", true);
                        break;
                        case 1:
                            current_note.tag ("marked_orange", true);
                        break;
                        case 2:
                            current_note.tag ("marked_yellow", true);
                        break;
                        case 3:
                            current_note.tag ("marked_green", true);
                        break;
                        case 4:
                            current_note.tag ("marked_blue", true);
                        break;
                        case 5:
                            current_note.tag ("marked_purple", true);
                        break;
                    }

                    current_note.changed ();
                }
            });

            var mark_clear_button = new Gtk.Button.with_label(_("Clear"));
            mark_clear_button.margin = 0;
            mark_clear_button.can_focus = false;
            mark_clear_button.hexpand = true;
            mark_clear_button.clicked.connect (() => {
                mark_color_buttons.selected = -1;
                current_note.clear_marked_color ();
                mark_popover.hide ();
            });


            var mark_grid_content = new Gtk.Grid ();
            mark_grid_content.column_spacing = 6;
            mark_grid_content.margin = 6;
            mark_grid_content.attach (mark_color_buttons, 0, 0, 1, 1);
            mark_grid_content.attach (mark_clear_button, 1, 0, 1, 1);

            mark_popover = new Gtk.Popover (mark);
            mark_popover.position = Gtk.PositionType.BOTTOM;
            mark_popover.add (mark_grid_content);

            revert = new Gtk.ToolButton (null, null);
            revert.label = _("revert");
            revert.icon_name = "document-revert";
            revert.tooltip_text = _("Restore this note");

            editbar.insert (revert,                       -1);
            editbar.insert (new Gtk.SeparatorToolItem (), -1);
            editbar.insert (undo,                         -1);
            editbar.insert (redo,                         -1);
            editbar.insert (new Gtk.SeparatorToolItem (), -1);
            editbar.insert (bold,                         -1);
            editbar.insert (italic,                       -1);
            editbar.insert (underline,                    -1);
            editbar.insert (strike,                       -1);
            editbar.insert (new Gtk.SeparatorToolItem (), -1);
            editbar.insert (mark,                         -1);

            content_scroll.add (this.content);
            content_scroll.hscrollbar_policy = Gtk.PolicyType.NEVER;
            this.content.wrap_mode = Gtk.WrapMode.WORD_CHAR;
            this.title.get_style_context ().add_class ("h2");
            this.title.margin_top = 12;
            this.title.style_updated ();

            content_scroll.margin_left = this.title.margin_left = 12;

            container.pack_start (editbar, false);
            container.pack_start (title, false);
            container.pack_start (content_scroll);

            this.title.margin_bottom = 12;

            this.add (container);

            /*
             * EVENTS
             */

            this.title.key_press_event.connect ( (e) => {
                if (e.keyval == Gdk.Key.Tab || e.keyval == Gdk.Key.Return) {
                    content.grab_focus ();
                    return true;
                }
                return false;
            });

            this.revert.clicked.connect ( () => {
                this.cancel ();
            });

            this.key_press_event.connect ( (e) => {
                if (e.keyval == Gdk.Key.s && Gdk.ModifierType.CONTROL_MASK in e.state) {
                    this.save ();
                    return true;
                }
                if (e.keyval == Gdk.Key.b && Gdk.ModifierType.CONTROL_MASK in e.state) {
                    bold.active = !bold.active;
                    return true;
                }
                if (e.keyval == Gdk.Key.i && Gdk.ModifierType.CONTROL_MASK in e.state) {
                    italic.active = !italic.active;
                    return true;
                }
                if (e.keyval == Gdk.Key.k && Gdk.ModifierType.CONTROL_MASK in e.state) {
                    strike.active = !strike.active;
                    return true;
                }
                if (e.keyval == Gdk.Key.u && Gdk.ModifierType.CONTROL_MASK in e.state) {
                    underline.active = !underline.active;
                    return true;
                }
                return false;
            });
        }

        public void set_focus () {
            if (title.buffer.text == _("Untitled") || title.buffer.text == _("New Note"))
                title.grab_focus ();
            else {
                content.grab_focus ();
                content_scroll.scroll_child (Gtk.ScrollType.END, false);
            }
        }

        public void save () {
            if (current_note != null && current_note.is_changed) {
                current_note.save ();
            }
        }

        private void cancel () {
            current_note.restore ();
        }

        public void open_note (Footnote.Objects.Note n) {
            this.save ();

            if (parent == null)
                open (); //show the edit view

            title.buffer = n.title;

            content.buffer = n.content;
            content.show_all ();
            content.buffer.notify["cursor-position"].connect ( () => {
                Gtk.TextIter s;
                content.buffer.get_selection_bounds (out s, null);
                string [] tags = new string[s.get_tags ().length ()];
                for (var i = 0;i < s.get_tags ().length (); i++) {
                    tags[i] = s.get_tags ().nth_data (i).name;
                }

                applying_style = true;
                bold.active = ("bold" in tags);
                italic.active = ("italic" in tags);
                strike.active = ("strikethrough" in tags);
                underline.active = ("underline" in tags);

                if ("marked_red" in tags)
                    mark_color_buttons.selected = 0;
                else if ("marked_orange" in tags)
                    mark_color_buttons.selected = 1;
                else if ("marked_yellow" in tags)
                    mark_color_buttons.selected = 2;
                else if ("marked_green" in tags)
                    mark_color_buttons.selected = 3;
                else if ("marked_blue" in tags)
                    mark_color_buttons.selected = 4;
                else if ("marked_purple" in tags)
                    mark_color_buttons.selected = 5;
                else
                    mark_color_buttons.selected = -1;
                applying_style = false;
            });

            this.revert.sensitive = false;
            current_note = n;

            current_note.restored.connect ( () => { this.revert.sensitive = false; });
            current_note.saved.connect ( () => { this.revert.sensitive = false; });
            current_note.changed.connect ( () => { this.revert.sensitive = true; });
        }

        public static Gdk.Pixbuf get_color_dot (double r, double g, double b, double a) {
            int w = 16;
            int h = 16;
            var surface = new Cairo.ImageSurface (Cairo.Format.ARGB32, w, h);
            var ctx     = new Cairo.Context (surface);

            var grad = new Cairo.Pattern.rgba (r / 255, g / 255, b / 255, a);

            ctx.set_line_width (1);

            Granite.Drawing.Utilities.cairo_rounded_rectangle (ctx, 0.5, 0.5, w-1, h-1, 1);
            ctx.set_source (grad);
            ctx.fill_preserve ();
            ctx.set_source_rgba (0, 0, 0, 0.3);
            ctx.stroke ();

            return Gdk.pixbuf_get_from_surface (surface, 0, 0, w, h);
        }
    }
}
