// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/*-
 * Copyright (c) 2011-2015 Footnote Developers (http://launchpad.net/footnote)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Corentin Noël <corentin@elementaryos.org>
 */

public class Footnote.Widgets.MonthBar : Gtk.Toolbar {
    public signal void month_changed (int month);
    public signal void year_changed (int year);
    public int month = -1;
    public int year = -1;
    private Gtk.ComboBoxText years_combobox;
    private Gtk.RadioToolButton[] month_buttons;
    public MonthBar () {
        icon_size = Gtk.IconSize.MENU;
        toolbar_style = Gtk.ToolbarStyle.TEXT;
        get_style_context ().add_class ("secondary-toolbar");
        month_buttons = new Gtk.RadioToolButton[12];

        var years_combobox_item = new Gtk.ToolItem ();
        years_combobox = new Gtk.ComboBoxText ();
        years_combobox_item.add (years_combobox);
        years_combobox_item.margin_right = 12;

        var all = new Gtk.RadioToolButton (null);
        all.label = _("All");
        all.toggled.connect (() => {
            this.month = -1;
            month_changed (-1);
        });

        insert (years_combobox_item, -1);
        insert (all, -1);
        all.set_homogeneous (false);

        var months = new Gee.LinkedList<string> ();
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_1));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_2));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_3));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_4));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_5));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_6));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_7));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_8));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_9));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_10));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_11));
        months.add (Posix.nl_langinfo (Posix.NLItem.MON_12));
        int i = 1;
        foreach (var month in months) {
            var w = new Gtk.RadioToolButton.from_widget (all);
            w.label = month;
            w.set_homogeneous (false);
            insert (w, -1);
            month_buttons[i-1] = w;
            var m = i;
            w.toggled.connect (() => {
                this.month = m;
                month_changed (m);
            });

            i++;
        }

        int old_combo_value = -1;
        years_combobox.changed.connect (() => {
            if (years_combobox.get_active_text () == null)
                return;

            if (years_combobox.active_id == "-1") {
                if (old_combo_value == -1)
                    return;

                this.year = -1;
                year_changed (-1);
                old_combo_value = -1;
            } else {
                var current_value = int.parse (years_combobox.get_active_text ());
                if (current_value == old_combo_value)
                    return;

                this.year = current_value;
                year_changed (current_value);
                old_combo_value = current_value;
            }
        });
    }

    public void generate_year_combobox (Gee.HashSet<int> years_set) {
        var current_active_id = years_combobox.active_id;
        years_combobox.remove_all ();
        years_combobox.append ("-1", _("Any"));

        foreach (var year in years_set) {
            years_combobox.append_text ("%d".printf (year));
        }

        if (current_active_id == null)
            years_combobox.active_id = "-1";
        else
            years_combobox.active_id = current_active_id;

        years_combobox.show_all ();
    }

    public void update_sensitivity (Gee.HashSet<int> months) {
        for (int i = 0; i < 12; i++) {
            if (months.contains (i))
                month_buttons[i].sensitive = true;
            else
                month_buttons[i].sensitive = false;
        }
    }
}
