// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/*-
 * Copyright (c) 2011-2015 Footnote Developers (http://launchpad.net/footnote)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Corentin Noël <corentin@elementaryos.org>
 */

public class Footnote.Widgets.SideBar : Granite.Widgets.SourceList {
    Granite.Widgets.SourceList.ExpandableItem notebooks_items;
    Granite.Widgets.SourceList.ExpandableItem starred_items;
    public SideBar () {
        notebooks_items = new Granite.Widgets.SourceList.ExpandableItem (_("Notebooks"));
        starred_items = new Granite.Widgets.SourceList.ExpandableItem (_("Bookmarks"));
        root.add (notebooks_items);
        root.add (starred_items);
        root.expand_all ();
    }

    public void add_notebook (Footnote.Objects.Notebook notebook) {
        var item = new NotebookItem (notebook);
        notebooks_items.add (item);
        foreach (var note in notebook.get_notes ()) {
            if (note.bookmarked) {
                add_bookmark (note);
            }
        }
        notebook.note_bookmark.connect ((note, is_bookmarked) => {
            if (is_bookmarked) {
                add_bookmark (note);
            } else {
                remove_bookmark (note);
            }
        });

        notebook.note_removed.connect ((note) => {
            if (note.bookmarked)
                remove_bookmark (note);
        });

        if (selected == null || notebook.id == Settings.get_default ().last_notebook)
            selected = item;
    }

    public void remove_notebook (Footnote.Objects.Notebook notebook) {
        notebooks_items.remove (notebook.sidebar_item);
        foreach (var n in notebook.get_notes ()) {
            if (n.bookmarked)
                remove_bookmark (n);
        }
    }
    
    public void select_notebook (Footnote.Objects.Notebook notebook) {
        if (selected is Widgets.BookmarkItem || 
            selected is Widgets.NotebookItem && (selected as Widgets.NotebookItem).notebook != notebook) {
            this.notebooks_items.children.@foreach ( (item) => {
                if ((item as NotebookItem).notebook == notebook) {
                    this.item_selected (item as NotebookItem);
                    return false;
                }
                return true;
            });
        }
    }

    public void add_bookmark (Footnote.Objects.Note note) {
        var item = new BookmarkItem (note);
        starred_items.add (item);
    }

    public void remove_bookmark (Footnote.Objects.Note note) {
        Footnote.Widgets.BookmarkItem item = null;
        foreach (var child in starred_items.children) {
            var child_item = (Footnote.Widgets.BookmarkItem)child;
            if (child_item.note == note) {
                item = child_item;
                break;
            }
        }

        warn_if_fail (item != null);
        starred_items.remove (item);
    }
}

public class Footnote.Widgets.NotebookItem : Granite.Widgets.SourceList.Item {
    public Footnote.Objects.Notebook notebook;
    public NotebookItem (Footnote.Objects.Notebook notebook) {
        base (notebook.title);
        this.notebook = notebook;
        notebook.sidebar_item = this;
        icon = notebook.icon;
        notebook.modified.connect (() => {
            name = notebook.title;
            icon = notebook.icon;
        });
    }
}

public class Footnote.Widgets.BookmarkItem : Granite.Widgets.SourceList.Item {
    public Footnote.Objects.Note note;
    public BookmarkItem (Footnote.Objects.Note note) {
        base (note.title.text);
        this.note = note;
        refresh_icon ();
        note.notebook.modified.connect (() => {
            refresh_icon ();
        });
        note.saved.connect (() => {
            name = note.title.text;
        });
    }

    private void refresh_icon () {
        try {
            var pixbuf = Gtk.IconTheme.get_default ().load_icon ("text-x-generic-symbolic", 16, Gtk.IconLookupFlags.GENERIC_FALLBACK);
            unowned uint8[] pixels = pixbuf.get_pixels ();
            int step = 3;
            if (pixbuf.has_alpha) {
                step = 4;
            }

            var pix_number = pixbuf.width * pixbuf.height * step;
            uint8[] new_pixels = new uint8[pix_number];
            for (int i = 0; i < pix_number; i+=step) {
                var r = pixels[i];
                var g = pixels[i+1];
                var b = pixels[i+2];
                r = (uint8)(((double)r)*note.notebook.color.red);
                g = (uint8)(((double)g)*note.notebook.color.green);
                b = (uint8)(((double)b)*note.notebook.color.blue);
                new_pixels[i] = r;
                new_pixels[i+1] = g;
                new_pixels[i+2] = b;
                if (pixbuf.has_alpha)
                    new_pixels[i+3] = pixels[i+3];
            }

            icon = new Gdk.Pixbuf.from_data (new_pixels, pixbuf.colorspace, pixbuf.has_alpha, 
                        pixbuf.bits_per_sample, pixbuf.width, pixbuf.height, pixbuf.rowstride);
        } catch (Error e) {
            critical (e.message);
        }
    }
}
